/*
 * Defines.cpp
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
/*
 * This file defines a bunch of macros and defines that the
 * C-preprocessor uses on .Java files when generating a .java file.
 *
 * See the Makefile (java/Makefiles/Preprocess.mk) for just how that
 * happens.
 *
 * Note: All .Java files are "dependent" on this file (in make terms)
 * so making a change here will cause the world to need recompilation.
 */

#define JANOS_ROOT_PKG                 edu.utah.janos
#define JANOS_NODEOS_PKG               JANOS_ROOT_PKG##.nodeos
#define JANOS_NATIVE_NODEOS_PKG        JANOS_ROOT_PKG##.nodeos
#define JANOS_IO_PKG                   JANOS_ROOT_PKG##.io
#define JANOS_UTIL_PKG                 JANOS_ROOT_PKG##.util
#define JANOS_ROUTE_PKG                JANOS_ROOT_PKG##.route
#define JANOS_TESTS_PKG                JANOS_ROOT_PKG##.tests
#define JANOS_TESTS_NODEOS_PKG         JANOS_ROOT_PKG##.tests.nodeos
#define JANOS_TESTS_ANTSR_CORE_PKG     JANOS_ROOT_PKG##.tests.antsr.core
#define JANOS_TESTS_ANTSR_UTIL_PKG     JANOS_ROOT_PKG##.tests.antsr.util

#define JANOS_ANTSR_CORE_PKG           JANOS_ROOT_PKG##.antsr.core
#define JANOS_ANTSR_CORE_WRAPPER_PKG   JANOS_ANTSR_CORE_PKG##.wrapper
#define JANOS_ANTSR_UTIL_PKG           JANOS_ROOT_PKG##.antsr.util
#define JANOS_ANTSR_ROUTE_PKG          JANOS_ROOT_PKG##.antsr.route

#define JANOS_ANTSR_APPS_PKG           JANOS_ROOT_PKG##.antsr.apps
#define JANOS_ANTSR_PING_APP_PKG       JANOS_ANTSR_APPS_PKG##.ping
#define JANOS_ANTSR_PONG_APP_PKG       JANOS_ANTSR_APPS_PKG##.pong
#define JANOS_ANTSR_MOBILE_APP_PKG     JANOS_ANTSR_APPS_PKG##.mobile
#define JANOS_ANTSR_MULTICAST_APP_PKG  JANOS_ANTSR_APPS_PKG##.multicast

#define JANOS_NODEOS_PKG_S              "edu.utah.janos.nodeos"
#define JANOS_NATIVE_NODEOS_PKG_S       "edu.utah.janos.nodeos"
#define JANOS_IO_PKG_S                  "edu.utah.janos.io"
#define JANOS_UTIL_PKG_S                "edu.utah.janos.util"
#define JANOS_ROUTE_PKG_S               "edu.utah.janos.route"
#define JANOS_TESTS_PKG_S               "edu.utah.janos.tests"
#define JANOS_TESTS_NODEOS_PKG_S        "edu.utah.janos.tests.nodeos"
#define JANOS_TESTS_ANTSR_CORE_PKG_S    "edu.utah.janos.tests.antsr.core"
#define JANOS_TESTS_ANTSR_UTIL_PKG_S    "edu.utah.janos.tests.antsr.util"

#define JANOS_ANTSR_CORE_PKG_S          "edu.utah.janos.antsr.core"
#define JANOS_ANTSR_CORE_WRAPPER_PKG_S  "edu.utah.janos.antsr.core.wrapper"
#define JANOS_ANTSR_UTIL_PKG_S          "edu.utah.janos.antsr.util"
#define JANOS_ANTSR_ROUTE_PKG_S         "edu.utah.janos.antsr.route"

#define JANOS_ANTSR_APPS_PKG_S          "edu.utah.janos.antsr.apps"
#define JANOS_ANTSR_PING_APP_PKG_S      "edu.utah.janos.antsr.apps.ping"
#define JANOS_ANTSR_PONG_APP_PKG_S      "edu.utah.janos.antsr.apps.pong"
#define JANOS_ANTSR_MOBILE_APP_PKG_S    "edu.utah.janos.antsr.apps.mobile"
#define JANOS_ANTSR_MULTICAST_APP_PKG_S "edu.utah.janos.antsr.apps.multicast"

/*
 * Convenience macros.
 */

#define UNSIGNED_BYTE2INT(x)	((x) & 0xff)

#define UNSIGNED_SHORT2INT(x)	((x) & 0xffff)

#define EXTRACT_SHORT(array, offset) 			\
	(UNSIGNED_SHORT2INT(				\
	 (UNSIGNED_BYTE2INT(array[(offset)]) << 8) | 	\
	 (UNSIGNED_BYTE2INT(array[(int)(offset)+1]))))

#define EXTRACT_INT(array, offset) 				\
	((UNSIGNED_BYTE2INT(array[(offset)]) << 24) | 		\
	 (UNSIGNED_BYTE2INT(array[(offset) + 1]) << 16) | 	\
	 (UNSIGNED_BYTE2INT(array[(offset) + 2]) << 8) | 	\
	 (UNSIGNED_BYTE2INT(array[(offset) + 3])))

/* Make all class state visible for testing */
#ifdef EXPOSED
# define private public
# define PACKAGE public
# define protected public
#endif

/* So we may label our package-private methods with a token */
#ifndef PACKAGE
# define PACKAGE 
#endif

#ifndef DEBUG
# ifndef NDEBUG
# error DEBUG or NDEBUG must be defined on the preprocessor command line
# endif
#endif

#ifdef DEBUG
# define DEBUG_ON true
# define IMPORT_DEBUG import JANOS_NODEOS_PKG.Debug;
# define IMPORT_TRACE import JANOS_NODEOS_PKG.Trace;
# define DEBUGONLY(x) x
# define assert(x) JANOS_NODEOS_PKG.Debug.assertCheck(x, #x, __FILE__, __LINE__)
# define assertLocked(x) JANOS_NODEOS_PKG.Debug.checkLocked(x, #x, __FILE__, __LINE__)
# define FATALEXCEPTION(EXEP, MSG) throw JANOS_NODEOS_PKG.Debug.fatalException(EXEP, MSG)
# define panic(STR) throw JANOS_NODEOS_PKG.Debug.dPanic(STR)
# define NOTIMPLEMENTED(MSG) throw JANOS_NODEOS_PKG.Debug.notImplemented(MSG)
/* XXX Make trace flags a per-flow variable. */
# define TRACE(FLAGS, STR) { if (JANOS_NODEOS_PKG.Trace.check(FLAGS)) { JANOS_NODEOS_PKG.Debug.pTrace(__FILE__, __LINE__, STR); } }
#endif

#ifdef NDEBUG
# define DEBUG_ON false
# define IMPORT_DEBUG 
# define IMPORT_TRACE 
# define DEBUGONLY(x)
# define assert(x)
# define assertLocked(x)
# define FATALEXCEPTION(EXEP, MSG) throw new InternalError(EXEP.getMessage() + MSG)
# define panic(STR) throw new InternalError(STR)
# define NOTIMPLEMENTED(MSG) throw new InternalError(MSG)
# define TRACE(x, y)
#endif

/* Make an attempt to look a bit more professional: */
#define require(x) assert(x)
#define ensure(x) assert(x)

#ifdef BENCH
# define BENCHONLY(x) x
#else
# define BENCHONLY(x)
#endif
