###
### Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
### All rights reserved.
###
### This file is part of Janos.
### 
### Janos is free software; you can redistribute it and/or modify it
### under the terms of the GNU General Public License, version 2, as
### published by the Free Software Foundation.
### 
### Janos is distributed in the hope that it will be useful, but
### WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details, in the file
### COPYING.  If that file is not present, write to the Free
### Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
### 02111-1307, USA
### 
### The University of Utah and the Flux Group requests users to
### return to flux-dist@cs.utah.edu any improvements that they make.
###

###
### Define a package for compilation
###
### Expects 'PACKAGE_NAME' to be defined.  Expects PACKAGE_NAME to
### 'match' (in the Java sense) a directory that contians the classes for
### that package.
###
### This makefile fragment must be included once for each package;
### it adds rules for preprocessing the package and compiling
### all of its sources.
###


# Package-specific directory names
$(PACKAGE_NAME)_FULLNAME:=$(PACKAGE_PREFIX).$(PACKAGE_NAME)
$(PACKAGE_NAME)_SRC_DIR:=$(subst .,/,$(PACKAGE_NAME))
$(PACKAGE_NAME)_DEST_DIR:=$(subst .,/,$($(PACKAGE_NAME)_FULLNAME))

# Find the various files related to this package
$(PACKAGE_NAME)_SOURCES_GLOB:=$(SRC_DIR)/$($(PACKAGE_NAME)_SRC_DIR)/*.Java
$(PACKAGE_NAME)_SOURCES:=$(wildcard $(SRC_DIR)/$($(PACKAGE_NAME)_SRC_DIR)/*.Java)
$(PACKAGE_NAME)_BASE_SOURCES:=$(subst $(SRC_DIR)/,,$($(PACKAGE_NAME)_SOURCES))
$(PACKAGE_NAME)_CPPSOURCES:=$(addprefix $(DEST_DIR)/$(PACKAGE_PREFIX_DIR)/, $($(PACKAGE_NAME)_BASE_SOURCES:.Java=.java))

# Minimum expected classes:
$(PACKAGE_NAME)_MIN_CLASSES:=$(sort $($(PACKAGE_NAME)_CPPSOURCES:.java=.class))


# Add the sources to the global list of all sources
ALL_SOURCES+=$($(PACKAGE_NAME)_SOURCES)
ALL_CPPSOURCES+=$($(PACKAGE_NAME)_CPPSOURCES)

# Post-processed source should be left around
.PRECIOUS: $($(PACKAGE_NAME)_CPPSOURCES)

## Generate a post-processed (.java) file from a .Java
## Depends on the file that defines the macros.  (NOTE 
## that the warning must not add any lines to the generated files)
## Also, note that this rule is independent of the package.
$(DEST_DIR)/$(PACKAGE_PREFIX_DIR)/%.java: $(SRC_DIR)/%.Java $(CPP_GLOBAL_MACROS)
	$(QUIET) set -e 
	@echo Preprocessing $<
	$(QUIET) $(RM) $@
	$(QUIET) $(ECHO_CLEAN) \
		"/* WARNING: generated from $<. -*- buffer-read-only: t -*- */" > $@
	$(QUIET) $(PREPROCESS) $< >> $@ || ($(RM) $@; $(FAILURE))

## Compile the package.  Sends all of the .java files to the compiler
## all at once.  Depends on the .java files all being up to date.  The
## .compile_FOO file is actually created.  If the compile fails, the
## error output is massaged so that it references the original .Java
## files (heh).
.compile_$(PACKAGE_NAME): $($(PACKAGE_NAME)_CPPSOURCES)
	$(QUIET) echo "Compiling $(subst .compile_,,$@) package"
	$(QUIET) $(RM) $@ $(@)-output
	$(QUIET) test -z "$$JIKESPATH" || echo "WARNING: JIKESPATH is set.  This may cause compilation errors!"
	$(QUIET) if test -z "$^"; then \
		echo "ERROR: No files to compile!"; \
		$(FAILURE); \
	fi
	$(QUIET) if $(COMPILE_JAVA) $^ 1> $(@)-output 2>&1 ; then \
		$(SED) -e 's/\.java/\.Java/g' \
	        	-e 's,$(DEST_DIR)/$(PACKAGE_PREFIX_DIR),$(SRC_DIR),g' $(@)-output; \
		touch $@;					  \
	else 							  \
		$(SED) -e 's/\.java/\.Java/g' \
		        -e 's,$(DEST_DIR)/$(PACKAGE_PREFIX_DIR),$(SRC_DIR),g' $(@)-output; \
		$(FAILURE);                                       \
	fi



$(PACKAGE_NAME)_EXPECTED_CPP:=$(sort $($(PACKAGE_NAME)_CPPSOURCES))
$(PACKAGE_NAME)_ACTUAL_CPP_GLOB:=$(DEST_DIR)/$($(PACKAGE_NAME)_DEST_DIR)/*.java
$(PACKAGE_NAME)_ACTUAL_CPP:=$(sort $(wildcard $($(PACKAGE_NAME)_ACTUAL_CPP_GLOB)))
$(PACKAGE_NAME)_EXTRA_CPP=$(strip $(filter-out  $($(PACKAGE_NAME)_EXPECTED_CPP),  $($(PACKAGE_NAME)_ACTUAL_CPP)))

.PHONY: sanitycheck-$(PACKAGE_NAME)

### Check for stale .java/.class files lying around in the output directories
sanitycheck-$(PACKAGE_NAME): $($(PACKAGE_NAME)_MIN_CLASSES)
	$(QUIET) if test -z "$($(subst sanitycheck-,,$@)_SOURCES)"; then \
	  echo "WARNING: Package $(subst sanitycheck-,,$@) contains no source files!?!?"; \
	fi
	$(QUIET) if test -n "$($(subst sanitycheck-,,$@)_EXTRA_CPP)"; then \
	  echo "WARNING:  Package $(subst sanitycheck-,,$@) destination contains: $(subst $(DEST_DIR)/,,$^)"; \
	fi

%.class:
	$(QUIET) echo "ERROR: Expected class file $@ is missing.  Check .compile_*-output files"
	$(FAILURE)


## Generate the package target directory if necesary
$(DEST_DIR)/$($(PACKAGE_NAME)_DEST_DIR):
	$(MKDIR) $@


## Add the directory to the list of required directories
DEST_DIRECTORIES+=$(DEST_DIR)/$($(PACKAGE_NAME)_DEST_DIR)

## Add the compile target to the list of package compile targets
COMPILATION_TARGETS+=.compile_$(PACKAGE_NAME) sanitycheck-$(PACKAGE_NAME)

###
### Cleanup target for this package
###

clean:: clean_$(PACKAGE_NAME)

clean_%::
	@echo Cleaning $*
	$(RM) $(DEST_DIR)/$($(*)_DEST_DIR)/*.java
	$(RM) $(DEST_DIR)/$($(*)_DEST_DIR)/*.class

###
### Debugging target for this package
###

debug:: debug_$(PACKAGE_NAME)

debug_%::
	@echo 
	@echo "**** Targets for $* package"
	@echo "    $* Fullname    = $($(*)_FULLNAME)"
	@echo "    $* SRC_DIR     = $($(*)_SRC_DIR)"
	@echo "    $* DEST_DIR    = $($(*)_DEST_DIR)"
	@echo "    $* SOURCES     = $($(*)_SOURCES)"
	@echo "    $* SOURCES_GLOB= $($(*)_SOURCES_GLOB)"
	@echo "    $* CPP sources = $($(*)_CPPSOURCES)"
	@echo "    $* EXPECTED_CPP sources = $($(*)_EXPECTED_CPP)"
	@echo "    $* ACTUAL_CPP sources = $($(*)_ACTUAL_CPP)"
	@echo "    $* EXTRA_CPP sources = $($(*)_EXTRA_CPP)"
	@echo "    $* ACTUAL_CPP_GLOB = $($(*)_ACTUAL_CPP_GLOB)"

