###
### Top-level Janos Java Makefile
###
### Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
### All rights reserved.
###
### This file is part of Janos.
### 
### Janos is free software; you can redistribute it and/or modify it
### under the terms of the GNU General Public License, version 2, as
### published by the Free Software Foundation.
### 
### Janos is distributed in the hope that it will be useful, but
### WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details, in the file
### COPYING.  If that file is not present, write to the Free
### Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
### 02111-1307, USA
### 
### The University of Utah and the Flux Group requests users to
### return to flux-dist@cs.utah.edu any improvements that they make.
###

### Run 'make showhelp' if you don't want to read this file.

###
### Variables that control what is built
###
# BUILD=[debug|optimized|exposed]   [debug]
# OBJ_DIR=<dir>               [`pwd`]
# SRC_DIR=<dir>               [`pwd`]
# VERBOSEMAKE=[yes|no]        [no]

###
### Provide defaults for command-line variables
###

ifndef BUILD
BUILD:=debug
endif

# The directory in which all output will be generated.
ifndef OBJ_DIR
OBJ_DIR:=$(shell pwd)
endif

# The top of the Janos Java source tree
ifndef SRC_DIR
SRC_DIR:=$(shell pwd)
endif

QUIET:=@
ifeq ($(VERBOSEMAKE),yes)
QUIET:=
endif

###
### "Parse" the command-line variables
###

# Default to stuff that blows up good.
LIBSTYLE:=ERROR

ifeq ($(BUILD),debug)
LIBSTYLE:=debug
endif

ifeq ($(BUILD),exposed)
LIBSTYLE:=exposed
endif

ifeq ($(BUILD),optimized)
LIBSTYLE:=optimized
endif

###
### Define some global values dependent upon the build
###

### Package prefix for all generated packages
PACKAGE_PREFIX:=edu.utah.janos
PACKAGE_PREFIX_DIR:=$(subst .,/,$(PACKAGE_PREFIX))

### destination directory for class files
DEST_DIR:=$(OBJ_DIR)/classes-$(BUILD)


###
### The default rule.  Must be defined before any includes.
###

ifeq ($(LIBSTYLE),ERROR)

# Trap bad Makefile flags here
default all: 
	@echo "FATAL ERROR in GNUmakefile Unknown BUILD flag '$(BUILD)'"

else

all: sanitycheck statusreport alldirectories allpackages

endif

###
### Include configuration defintions.
###

# Defines things like ECHO_CLEAN, MKDIR, etc.
include $(SRC_DIR)/Makefiles/System.mk

# Define the PREPROCESS magic
include $(SRC_DIR)/Makefiles/Preprocess.mk

# Define the COMPILE_JAVA command
include $(SRC_DIR)/Makefiles/Java.mk

###
### Include all of the package descriptions.  Include them in
### dependency order.  (One lame restriction of this model is that
### there cannot be ciruclar dependencies between packages, there must
### be a strict partial ordering.)
###

## Clear these variables.  They will be added to by each include of
## DefinePackage.
ALL_SOURCES:=
COMPILATION_TARGETS:=
DEST_DIRECTORIES:=

PACKAGE_NAME:=nodeos
include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=io
include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=route
include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=util
include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=antsr.util
include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=antsr.core
include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=antsr.route
include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=antsr.apps.ping
include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=antsr.apps.pong
include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=antsr.apps.mobile
include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=antsr.apps.multicast
include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=tests
include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=tests.nodeos
include $(SRC_DIR)/Makefiles/DefinePackage.mk

PACKAGE_NAME:=tests.antsr.core
include $(SRC_DIR)/Makefiles/DefinePackage.mk

###
### top-level rules
###

# A sanity check of SRC_DIR
SANITYCHECKFILE:=$(SRC_DIR)/nodeos/Main.Java
sanitycheck::
	@test -r $(SANITYCHECKFILE) \
		|| ( echo "Sanity check failed because $(SANITYCHECKFILE) is not readable."; \
			false; )

# A one-line blurb about what is actually being built.
statusreport:
	@echo "Building the Janos Java Libraries."
	@echo "BUILD=$(BUILD);  SRC_DIR='$(SRC_DIR)';  DEST_DIR='$(DEST_DIR)'"


alldirectories: $(DEST_DIRECTORIES)
	@echo All directories exist.


allpackages: $(COMPILATION_TARGETS)
	@echo All packages are up to date.


### Some "online" documentation.
showhelp:
	@echo "Janos Java Bits Construction Kit:"
	@echo "  make [BUILD=<debug|optimized|exposed>] [all|debug|clean]"
	@echo " "
	@echo "  Variables:"
	@echo "    BUILD controls the type of .class files built (optimized, debugging, etc)."
	@echo "      (exposed means all fields and methods are public.  Can be handy.)"
	@echo " "
	@echo "  Targets:"
	@echo "    The 'all' target (default) builds all the classes."
	@echo "    The 'debug' target prints gobs of debugging info *about the makefiles*"
	@echo "       (try: 'debug_Java.mk', 'debug_System.mk', 'debug_ants.utils', etc.)."
	@echo "    The 'clean' target removes all of the generated .java and .class files."
	@echo "    The 'showhelp' target is not documented."


# All the .mk files should be available.  Otherwise, something is broken
%.mk:
	@echo FATAL ERROR: Required makefile fragment $@ is not available.
	$(FAILURE)

### Generic clean target (nothing to do at this level)
clean::

distclean::
	$(RM) $(OBJ_DIR)/.compile-*
	$(RM) -rf $(OBJ_DIR)/classes-debug
	$(RM) -rf $(OBJ_DIR)/classes-optimized
	$(RM) -rf $(OBJ_DIR)/classes-exposed
	@for file in $(DISTCLEAN_FILES); do \
		echo "$(RM) $$file"; \
		$(RM) $$file; \
	done

### Generic debug target  (all include makefiles have a debug:: target)
debug::  debug_Main.mk

debug_Main.mk::
	@echo 
	@echo "**** Top-level make:"
	@echo "    ALL_SOURCES = $(ALL_SOURCES)"
	@echo "    ALL_CPPSOURCES = $(ALL_CPPSOURCES)"
	@echo "    COMPILATION_TARGETS = $(COMPILATION_TARGETS)"
	@echo "    SRC_DIR             = $(SRC_DIR)"
	@echo "    OBJ_DIR             = $(OBJ_DIR)"
	@echo "    DEST_DIR            = $(DEST_DIR)"
	@echo "    PACKAGE_PREFIX_DIR  = $(PACKAGE_PREFIX_DIR)"
	@echo "    DEST_DIRECTORIES    = $(DEST_DIRECTORIES)"


# Most of this makefile is phony.
.PHONY: debug all clean distclean alldirectories allpackages sanitycheck statusreport showhelp

# eof
