/*
 * CodeRequestDemultiplexKey.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_ANTSR_CORE_PKG;

import JANOS_NODEOS_PKG.DemultiplexKey;
import JANOS_NODEOS_PKG.Flow;

/**
 * Match ANTSR packets for a specific ANTSR protocol plus an
 * additional MID after the standard ANTSR header.  Used to
 * match MIDIDRequest, DLRequest, and DLResponse capsules.
 */
PACKAGE class CodeRequestDemultiplexKey
	extends ANTSRDemultiplexKey
{
	private final int mid2Length;
	
	// ANTSRDemultiplexKey will match mid1 (the MID from Capsule.Java),
	// and then we have to skip mid2Skip bytes to get to mid2
	// (the MID from MIDIDRequest/DLRequest/DLResponse):
	private static final int mid2Skip =
		Capsule.HEADERLENGTH - Xdr.TYPEID;

	PACKAGE CodeRequestDemultiplexKey(Flow flow,
					  byte mid1Bytes[],
					  byte mid2Bytes[])
	{
		super(flow, mid1Bytes);
		this.mid2Length = mid2Bytes.length;
		this.match(mid2Skip, mid2Bytes.length, mid2Bytes, null);
	}
	
	public int whereDoesTheCapsuleDecodingActuallyStart(int payloadStart)
	{
		/* Back up over mid2 to the start of the payload */
		return payloadStart - mid2Skip - this.mid2Length;
	}
}
