/*
 * FlowStartSpec.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;


/**
 * Various attributes of a flow that must be defined before a
 * Flow can be created.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public class FlowStartSpec
{
	PACKAGE final ClassSpec[] classSpecs;
	PACKAGE final String startClass;
	PACKAGE final String[] args;
	PACKAGE final CommSpaceHandle commSpaceHandle;

	public FlowStartSpec(ClassSpec[] classSpecs,
			     String startClass,
			     String[] args,
			     CommSpaceHandle commSpaceHandle)
	{
		assert(classSpecs);
		assert(startClass);
		// XXX don't assert(args);
		// XXX don't assert(commSpaceHandle);

		this.classSpecs = classSpecs;
		this.startClass = startClass;
		this.args = args;
		this.commSpaceHandle = commSpaceHandle;

	}

	public String toString()
	{
		String retval;
		int lpc;
		
		retval = "FlowStartSpec[startClass="
			+ startClass
			+ "classSpecs={ ";
		for( lpc = 0; lpc < classSpecs.length; lpc++ )
		{
			if( lpc > 0 )
				retval += ", ";
			retval += classSpecs[lpc];
		}
		retval += " }; args={ ";
		for( lpc = 0; lpc < args.length; lpc++ )
		{
			if( lpc > 0 )
				retval += ", ";
			retval += args[lpc];
		}
		retval += " }]";
		return retval;
	}
}
