/*
 * InfoHook.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

import java.util.Hashtable; // XXX
import java.util.Enumeration; // XXX
import java.util.NoSuchElementException; // XXX
import java.util.StringTokenizer; // XXX

/**
 * A little class used for exporting information about state, data, etc...
 * <p>
 * XXX ideally, most of this status stuff should be external
 * to the nodeos core package.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public class InfoHook
{
	/**
	 * A symbolic name used to distinguish one hook from another
	 */
	private final String name;

	/**
	 * An InfoHook can have a set of children, allowing the user to narrow
	 * their choices based on a certain child.
	 */
	protected Hashtable children = new Hashtable();
	
	/**
	 * Constructor that sets the hooks name
	 */
	public InfoHook(String name)
	{
		this.name = name;
	}

	/**
	 * Get the name for this hook
	 */
	public final String getName()
	{
		return this.name;
	}

	/**
	 * Find a child with the given name.  The name can be fully qualified
	 * to search through a tree of InfoHooks.
	 */
	public final InfoHook findChild(String name)
	{
		StringTokenizer st = new StringTokenizer(name, ".");
		InfoHook retval = null;
		InfoHook curr;
		String childName;

		curr = this;
		try
		{
			childName = st.nextToken();
			do
			{
				curr = (InfoHook)curr.children.get(childName);
				childName = st.nextToken();
			} while( curr != null );
			/* The child wasn't found, the name is bad */
			retval = null;
		}
		catch(NoSuchElementException e)
		{
			/* We ran out of tokens, so curr is the wanted child */
			retval = curr;
		}
		return retval;
	}

	/**
	 * Add a child InfoHook to this hook
	 */
	public final void addChild(InfoHook ih)
	{
		synchronized (children) {
			this.children.put(ih.getName(), ih);
		}
	}

	/**
	 * Remove a child InfoHook
	 */
	public final void remChild(String name)
	{
		synchronized (children) {
			this.children.remove(name);
		}
	}
	
	/**
	 * Return a string array listing the names of the children
	 */
	public String[] getChildList()
	{
		Enumeration e;
		String retval[];
		int i;
		
		synchronized(children)
		{
			e = children.elements();
			retval = new String[children.size()];
			for( i = 0; i < children.size(); i++ )
			{
				assert(e.hasMoreElements());
				retval[i] = ((InfoHook)e.nextElement()).getName();
			}
			return retval;
		}
	}
	
	/**
	 * This returns an array of strings that report whatever information
	 * is interesting...
	 */
	public String[] getInfo()
	{
		return this.getChildList();
	}

}
