/*
 * Node.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

import java.lang.System;
import java.lang.Thread;

import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * The Node interface used by EEs.
 * <p>
 * The Node interface provides wrappers for the basic services
 * expected by EE runtimes running on Janos.
 * <p>
 * Basically a wrapper for java.lang.System facilities.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public final class Node
{
	public static void println(String msg)
	{
		java.lang.Thread.currentThread().setPriority(10); // XXX
		java.lang.System.out.println(msg);
		java.lang.Thread.currentThread().setPriority(5); // XXX
	}

	public static void print(String msg)
	{
		java.lang.Thread.currentThread().setPriority(10); // XXX
		java.lang.System.out.print(msg);
		java.lang.Thread.currentThread().setPriority(5); // XXX
	}
	
	public static void fatalError(String msg)
	{
		println(msg);
		exit(11);
	}

	public static void shutdown()
	{
		println("Shutting down.");
		exit(0);
	}

	private static void exit(int cond)
	{
		System.out.flush();
		System.err.flush();
		System.exit(cond);
	}

	public static long time()
	{
		return System.currentTimeMillis();
	}

	public static String hostname()
	{
		String hostname = "localhost";
		try
		{
			hostname = InetAddress.getLocalHost().getHostName();
		}
		catch(java.net.UnknownHostException _)
		{
			// ignore, hostname is set to default already
		}

		return hostname;
	}
}

