/*
 * ProtocolSpec.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

import java.util.Vector;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;

/**
 * NodeOS protocol spec parser, it takes formatted strings and splits apart
 * their protocol components.
 */
public class ProtocolSpec
{
	/**
	 * Contains the strings that make up the protocol format
	 */
	private final Vector components = new Vector();

	/**
	 * Exception indicating an exception in the format string for the
	 * protocol spec
	 */
	public static class FormatException
		extends Exception
	{
	}

	/**
	 * Construct an empty spec
	 */
	PACKAGE ProtocolSpec()
	{
	}

	/**
	 * Construct a spec from a formatted string
	 */
	PACKAGE ProtocolSpec(String spec)
		throws FormatException
	{
		StringTokenizer st = new StringTokenizer(spec, "/");
		
		try
		{
			String protocol;
			
			while( true )
			{
				protocol = st.nextToken();
				this.components.addElement(protocol);
			}
		}
		catch(NoSuchElementException e)
		{
		}
	}

	/**
	 * Return the number of components in the protocol spec
	 */
	PACKAGE int componentCount()
	{
		return this.components.size();
	}

	/**
	 * Return the protocol string for a specific index
	 */
	PACKAGE String getComponent(int index)
	{
		return (String)this.components.elementAt(index);
	}

	/**
	 * Add a protocol string component
	 */
	PACKAGE void addComponent(String protocol)
	{
		this.components.addElement(protocol);
	}

	public boolean equals(Object o)
	{
		boolean retval = false;
		
		if( o instanceof ProtocolSpec )
		{
			ProtocolSpec ps = (ProtocolSpec)o;
			int thisSize, psSize;

			thisSize = this.components.size();
			psSize = ps.components.size();
			if( thisSize == psSize )
			{
				int lpc;
				
				for( retval = true, lpc = 0;
				     (lpc < thisSize) && retval;
				     lpc++ )
				{
					retval = this.components.
						elementAt(lpc).equals(
							ps.components.
							elementAt(lpc));
				}
			}
		}
		return retval;
	}

	public String toString()
	{
		String retval = "";
		int lpc, size;

		size = this.components.size();
		for( lpc = 0; lpc < size; lpc++ )
		{
			if( lpc > 0 )
				retval += "/";
			retval += this.components.elementAt(lpc);
		}
		return retval;
	}
}
