/*
 * UdpTest.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

package JANOS_TESTS_PKG;

import JANOS_NODEOS_PKG.FlowRun;
import JANOS_NODEOS_PKG.Flow;
import JANOS_NODEOS_PKG.ThreadPool;
import JANOS_NODEOS_PKG.MemSpec;
import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.OutChannel;
import JANOS_NODEOS_PKG.InChannel;
import JANOS_NODEOS_PKG.DeliverFunc;
import JANOS_NODEOS_PKG.DemultiplexKey;
import JANOS_NODEOS_PKG.BufferHandle;
import JANOS_NODEOS_PKG.Address;
import JANOS_NODEOS_PKG.AddressFactory;
import JANOS_NODEOS_PKG.NoSuchNodeAddress;
import JANOS_UTIL_PKG.FlowBase;
import JANOS_UTIL_PKG.SimpleDemultiplexKey;

import java.net.InetAddress;
import java.io.IOException;

public class UdpTest
	extends FlowBase
{
	public UdpTest()
	{
	}

	static final int port1 = 33001;
	static final int port2 = 33002;
	static final byte[] key1 = {5, 4, 3, 2};
	static final byte[] key2 = {5, 4, 3, 1};
	static final int count = 100;

	public void start()
	{
		try 
		{
			start1();
		}
		catch(Throwable e)
		{
			e.printStackTrace();
			throw new RuntimeException("test failed");
		}
	}

	public void start1()
		throws Exception
	{
		AddressFactory af = Address.getAddressFactory();
		String iAddr = InetAddress.getLocalHost().getHostAddress();
		Address addr1 = af.newAddress(1,
					      "if/ipv4/udp",
					      "*:" + port1 + ":localhost");
		Address addr2 = af.newAddress(2,
					      "if/ipv4/udp",
					      "*:" + port2 + ":localhost");

		OutChannel out1 = makeOutChannel(addr2);
		OutChannel out2 = makeOutChannel(addr1);
		InChannel in1 = makeInChannel(addr1, out1, key1, key2);
		InChannel in2 = makeInChannel(addr2, out2, key2, key1);
		in1.addBuffers(10);
		in2.addBuffers(10);

		BufferHandle b = new BufferHandle(this.flow);
		b.newBuffer(key2.length + 4);
		b.setBytes(4, key2, 0, key2.length);

		out1.send(b);

		recycleBufferHandle(b);
	}

	private OutChannel makeOutChannel(Address outAddr)
		throws Exception
	{
		return new OutChannel(
			flow,
			outAddr.getProtocolSpec(),
			outAddr.getAddressSpec());
	}

	private InChannel makeInChannel(final Address inAddr,
					final OutChannel replyChannel,
					final byte[] recvKey,
					final byte[] replyKey)
		throws Exception
	{
		final byte[] data = new byte[replyKey.length + 4];
		InChannel retval;
		System.arraycopy(replyKey, 0, data, 4, replyKey.length);

		DeliverFunc deliver = new DeliverFunc()
		{
			// XXX this only works if packets are
			// reliable and ordered

			public void receive(BufferHandle packet, int _)
			{
				try {receive1(packet);}
				catch(IOException e) {e.printStackTrace();}
			}

			public void receive1(BufferHandle packet)
				throws IOException
			{
				int i = packet.getInt(0);

				if(i <= count)
				{
					Node.println("port "
						     + inAddr.getProtocolSpec()
						     + ";"
						     + inAddr.getAddressSpec()
						     + " recv:" + i);
					packet.setInt(0, i+1);
					packet.setBytes(4, replyKey, 0, replyKey.length);
					replyChannel.send(packet);
				}

				if(i >= count)
				{
					UdpTest.this.flow.destroy();
				}
			}
		};

		retval = new InChannel(flow,
				       new SimpleDemultiplexKey(this.flow,
								4,
								recvKey),
				       inAddr.getProtocolSpec(),
				       inAddr.getAddressSpec(),
				       threadPool,
				       deliver);
		return retval;
	}
}
