/*
 * DemultiplexKeyTester.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

package JANOS_TESTS_NODEOS_PKG;

import JANOS_TESTS_PKG.*;
import JANOS_NODEOS_PKG.*;

public class DemultiplexKeyTester extends Tester
{
	/**
	 * Generate a random array of bytes for a key
	 *
	 * @param length The length of the array
	 * @return An array of bytes with random element values
	 */
	private byte[] generateKeyBytes(int length)
	{
		byte retval[] = new byte[length];
		int lpc;

		for( lpc = 0; lpc < length; lpc++ )
		{
			retval[lpc] = (byte)(Math.random() * 10);
		}
		return retval;
	}

	/**
	 * Create an array from a section of another
	 *
	 * @param key The array to create the slice from
	 * @param offset The start of the section in key to copy
	 * @param length The length of the section in key to copy
	 * @return A new byte array that has the values from the section
	 * of key specified by the parameters
	 */
	private byte[] generateSlice(byte key[], int offset, int length)
	{
		byte retval[] = new byte[length];

		System.arraycopy(key, offset, retval, 0, length);
		return retval;
	}

	/**
	 * Create a byte array that won't match the specified subsection of key
	 *
	 * @param key The byte array to generate the bad slice from
	 * @param offset The start of the section in key to create the bad
	 * slice from
	 * @param length The length of the section in key
	 * @return A byte array that won't match the section in key specified
	 * by the parameters
	 */
	private byte[] generateBadSlice(byte key[], int offset, int length)
	{
		byte retval[] = new byte[length];
		int lpc;

		for( lpc = 0; lpc < length; lpc++ )
		{
			retval[lpc] = (byte)(key[offset + lpc] + 1);
		}
		return retval;
	}
	
	public void testUnit() throws TestFailedException
	{
		byte keyBytes[], keySlice[], badSlice[];
		int keyLength = 16;
		DemultiplexKey dk;
		int lpc;

		keyBytes = this.generateKeyBytes(keyLength);
		dk = new DemultiplexKey(TestMain.getRootFlow());
		dk.match(0, keyLength, keyBytes, null);
		this.enterSection(ENV_EXPOSED, "Accessor");
#ifdef EXPOSED
		this.test(dk.length() == keyLength);
#endif
		
		this.enterSection(0, "Comparisons");
		this.test(dk.matches(keyBytes, 0, keyLength) == keyLength);
		this.test(dk.matches(keyBytes, 0, keyLength - 1) == 0);
		this.test(dk.matches(keyBytes, 1, keyLength - 1) == 0);
		this.test(dk.matches(this.generateBadSlice(keyBytes,
							   0,
							   keyLength),
				     0,
				     keyLength) == 0);
	}
}
