/*
 * InChannelTester.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

package JANOS_TESTS_NODEOS_PKG;

import java.net.*;
import java.io.IOException;
import JANOS_TESTS_PKG.*;
import JANOS_NODEOS_PKG.*;
import JANOS_UTIL_PKG.SimpleDemultiplexKey;

public class InChannelTester extends Tester
{
	/**
	 * Keys for testing
	 */
	private final byte key[][] =
	{
		{ 0, 1, 2, 4 },
		{ 8, 16, 32, 64, 127, -128, -64 },
		{ 0 },
		{ 8, 16 },
		{ 5, 4, 3, 2, 1 }
	};
	private final InetAddress localhost;
	private final int port = 30000;
	private DatagramSocket socket;
	private final int packets = 100;
	private int packetsSent[] = new int[key.length];
	private int packetsLost = 0;
	private int packetsDelivered[] = new int[key.length];

	public InChannelTester()
		throws IOException
	{
		this.localhost = InetAddress.getLocalHost();
		this.socket = new DatagramSocket();
	}
	
	private void sendPacket(byte data[])
	{
		try
		{
			DatagramPacket dp = new DatagramPacket(data,
							       data.length,
							       this.localhost,
							       this.port);
			
			this.socket.send(dp);
		}
		catch(IOException e)
		{
			this.packetsLost++;
		}
	}

	private class Receiver implements DeliverFunc
	{
		private final int key;
		
		public Receiver(int key)
		{
			this.key = key;
		}

		public void receive(BufferHandle packet, int payloadOffset)
		{
			InChannelTester.this.packetsDelivered[key]++;
			packet.reset();
		}
	}
	
	public void testUnit() throws TestFailedException
	{
		SimpleDemultiplexKey sdk;
		InChannelHandle ich[];
		DeliverFunc df;
		int maxKeySize;
		int lpc;

		this.enterSection(ENV_NONE, "Packet Handling");
		ich = new InChannelHandle[key.length];
		maxKeySize = 0;
		/* Make some new channels and receiver threads */
		try
		{
			for( lpc = 0; lpc < key.length; lpc++ )
			{
				InChannel ic;
				
				maxKeySize = Math.max(maxKeySize, key[lpc].length);
				sdk = new SimpleDemultiplexKey(TestMain.
							       getRootFlow(),
							       0,
							       key[lpc]);
				df = new Receiver(lpc);
				ic = new InChannel(
					TestMain.getRootFlow(),
					sdk,
					"if/ipv4/udp",
					"*:" + port + ":" + this.localhost.getHostName(),
					TestMain.getRootFlow().threadPool(),
					df);
				ic.addBuffers(10);
				ich[lpc] = new InChannelHandle(ic);
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw new TestFailedException(this);
		}
		/* Send some packets through and make sure they're received */
		try
		{
			BufferHandle bh;
			int packetsLeft;
			int currKey;

			bh = new BufferHandle(TestMain.getRootFlow());
			bh.newBuffer(maxKeySize);
			for(packetsLeft = packets;
			    packetsLeft > 0;
			    packetsLeft--)
			{
				currKey = packetsLeft % key.length;
				this.sendPacket(this.key[currKey]);
				this.packetsSent[currKey]++;
				bh.setBytes(0, this.key[currKey],
					    0, this.key[currKey].length);
				ich[currKey].push(
					new BufferHandle(TestMain.
							 getRootFlow(), bh),
					0);
				this.packetsSent[currKey]++;
				java.lang.Thread.sleep(50);
			}
		}
		catch(InterruptedException e)
		{
		}
		for( lpc = 0; lpc < key.length; lpc++ )
		{
			if( this.packetsSent[lpc] !=
			    this.packetsDelivered[lpc] )
			{
				Node.println("Warning: Some packets weren't "
					     + "delivered for key number "
					     + lpc
					     + " although this may just be"
					     + " due to a drop by the OS");
				Node.println("  Packets sent: "
					     + this.packetsSent[lpc]
					     + "  delivered: "
					     + this.packetsDelivered[lpc]);
			}
		}
	}
}
