/*
 * OutChannelTester.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

package JANOS_TESTS_NODEOS_PKG;

import java.net.DatagramSocket;
import java.net.DatagramPacket;
import java.io.IOException;

import JANOS_TESTS_PKG.*;
import JANOS_NODEOS_PKG.*;
import JANOS_UTIL_PKG.SimpleDemultiplexKey;

public class OutChannelTester extends Tester
{
	private final String host = "localhost";
	private int port;
	
	private final int channels = 5;
	private final byte messages[][] = new byte[10][];

	private final int packets = 100;
	private int packetsSent[] = new int[messages.length];
	private int packetsDelivered[] = new int[messages.length];

	/**
	 * Runnable class used to receive packets
	 */
	private class Receiver implements Runnable
	{
		private DatagramSocket ds;
		private OutChannelTester oct;
		private final SimpleDemultiplexKey exitKey =
			new SimpleDemultiplexKey(TestMain.getRootFlow(),
						 0,
						 InChannel.exitPacket);

		public Receiver()
			throws IOException
		{
			this.ds = new DatagramSocket();
			OutChannelTester.this.port = this.ds.getLocalPort();
		}

		private boolean handlePacket(DatagramPacket dp)
		{
			boolean retval = false;
			byte data[] = dp.getData();

			if( this.exitKey.matches(data, 0, data.length) >= 0 )
				retval = true;
			return retval;
		}

		public void run()
		{
			try
			{
				final int packetSize = 8192;
				DatagramPacket dp;

				dp = new DatagramPacket(new byte[packetSize],
							packetSize);
				boolean done = false;
				
				while( !done )
				{
					this.ds.receive(dp);
					done = this.handlePacket(dp);
				}
			}
			catch(IOException e)
			{
			}
		}
	}

	/**
	 * Generates the messages to be sent, just a bunch of random bytes
	 */
	private void generateMessages()
	{
		int lpc, lpc2;

		for( lpc = 0; lpc < messages.length; lpc++ )
		{
			messages[lpc] = new byte[lpc + 2];
			for( lpc2 = 0; lpc2 < (lpc + 2); lpc2++ )
			{
				messages[lpc][lpc2] =
					(byte)(Math.random() * 10);
			}
		}
	}

	/**
	 * Send a message over the given out channel
	 */
	private void sendMessage(OutChannel oc, byte message[])
		throws Exception
	{
		BufferHandle bh;

		bh = new BufferHandle(TestMain.getRootFlow());
		bh.newBuffer(message.length);
		bh.setBytes(0, message, 0, message.length);
		oc.send(bh);
	}
	
	public void testUnit() throws TestFailedException
	{
		JANOS_NODEOS_PKG.Thread receiver;
		BufferHandle bh;
		OutChannel oc;
		Address addr;
		int anAddr;
		int lpc;

		this.enterSection(ENV_NONE, "Construction/sending");
		this.generateMessages();
		try
		{
			receiver = TestMain.getRootFlow().threadPool().
				threadStart(new Receiver());
			anAddr = Address.fromString("18.1.0.2");
			addr = TestMain.getAddressFactory().
				newAddress(anAddr,
					   "if/ipv4/udp",
					   "*:" + this.port + ":localhost");
			oc = new OutChannel(TestMain.getRootFlow(),
					    addr.getProtocolSpec(),
					    addr.getAddressSpec());
			for( lpc = 0; lpc < messages.length; lpc++ )
			{
				this.sendMessage(oc, messages[lpc]);
			}
			this.sendMessage(oc, InChannel.exitPacket);
		}
		catch(Exception e)
		{
			throw new TestFailedException(this);
		}
	}
}
